package com.shigongli.controller;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaJscode2SessionResult;
import com.shigongli.common.*;
import com.shigongli.constants.StatusConstant;
import com.shigongli.entity.SysUser;
import com.shigongli.entity.TaPerson;
import com.shigongli.entity.TaShopKeeper;
import com.shigongli.service.ISysUserService;
import com.shigongli.service.ITaPersonService;
import com.shigongli.service.ITaShopKeeperService;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpOAuth2AccessToken;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Api(tags = "登入/登出")
@RestController
public class LoginController extends BaseController {

    @Autowired
    ISysUserService iSysUserService;

    @Autowired
    ITaPersonService iTaPersonService;

    @Autowired
    ITaShopKeeperService iTaShopKeeperService;

    @Autowired
    WxUtils wxUtils;

    /**
     * 公众号登录
     * @param code
     * @return
     * @throws Exception
     */
    @PostMapping("/mp/login")
    @ApiOperation(value="登录", notes = "登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean mpLogin(@ApiParam("登录参数") @RequestParam("code") String code) throws Exception {
        String openid;
        WxMpUser wxMpUser = null;

        if (StringUtils.isEmpty(code)) {
            return ResponseBean.error("参数 code 不能为空", ResponseBean.ERROR_MISSING_PARAMS);
        }

        if ("test".equals(code)) {
            openid = code;
        } else {
            WxMpService mpService = wxUtils.getMpService();
            WxMpOAuth2AccessToken wxMpOAuth2AccessToken = mpService.oauth2getAccessToken(code);
            wxMpUser = mpService.oauth2getUserInfo(wxMpOAuth2AccessToken, null);
            openid = wxMpOAuth2AccessToken.getOpenId();
        }


        SysUser sysUser = iSysUserService.getByOpenId(openid);
        if (null == sysUser && null != wxMpUser) {
            sysUser = new SysUser();
            sysUser.setName(wxMpUser.getNickname());
            sysUser.setOpenid(wxMpUser.getOpenId());
            iSysUserService.save(sysUser);
        }

        if (StringUtils.isEmpty(sysUser.getPhone())) {
            return ResponseBean.error("请先联系管理员进行授权操作", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        // 生成 token
        Map<String, Object> claims = new HashMap<>();
        claims.put("userId", sysUser.getUserId());
        String jws = JWTUtils.encode(claims);
        sysUser.setToken(jws);

        return ResponseBean.success(sysUser);
    }

    /**
     * 小程序登录
     * @param code
     * @return
     * @throws Exception
     */
    @PostMapping("/ma/login")
    @ApiOperation(value="登录", notes = "登录", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean maLogin(@ApiParam("登录参数") @RequestParam("code") String code) throws Exception {
        String openid;
        WxMaJscode2SessionResult sessionInfo = null;

        if (StringUtils.isEmpty(code)) {
            return ResponseBean.error("参数 code 不能为空", ResponseBean.ERROR_MISSING_PARAMS);
        }

        if ("test".equals(code)) {
            openid = code;
        } else {
            WxMaService maService = wxUtils.getMaService();
            sessionInfo = maService.getUserService().getSessionInfo(code);
            openid = sessionInfo.getOpenid();
        }

        TaPerson taPerson = iTaPersonService.getByOpenId(openid);
        if (null == taPerson || StatusConstant.DELETE.equals(taPerson.getStatus())) {
            taPerson = new TaPerson();
            taPerson.setOpenid(openid);
            iTaPersonService.save(taPerson);
        }

        if (null != sessionInfo) {
            taPerson.setSessionKey(sessionInfo.getSessionKey());
        }

        List<TaShopKeeper> shopKeeperList = iTaShopKeeperService.getByPerson(taPerson.getPersonId());
        if (null != shopKeeperList && shopKeeperList.size() > 0) {
            taPerson.setShopKeeperList(shopKeeperList);
        }

        // 生成 token
        Map<String, Object> claims = new HashMap<>();
        claims.put("userId", taPerson.getPersonId());
        String jws = JWTUtils.encode(claims);
        taPerson.setToken(jws);

        return ResponseBean.success(taPerson);
    }
}
