package com.shigongli.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.shigongli.common.StringUtils;
import com.shigongli.constants.StatusConstant;
import com.shigongli.entity.TaHouse;
import com.shigongli.entity.TaPerson;
import com.shigongli.entity.TaShopKeeper;
import com.shigongli.mapper.TaHouseMapper;
import com.shigongli.service.ITaHouseService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.shigongli.service.ITaShopKeeperService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 房源 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-12-11
 */
@Service
public class TaHouseServiceImpl extends ServiceImpl<TaHouseMapper, TaHouse> implements ITaHouseService {

    @Autowired
    TaHouseMapper taHouseMapper;

    @Autowired
    ITaShopKeeperService iTaShopKeeperService;

    @Override
    public IPage<TaHouse> getHousesOfKeeper(IPage<TaHouse> pg, TaPerson taPerson) {

        return taHouseMapper.getHousesOfKeeper(pg, taPerson.getPersonId());
    }

    @Override
    public boolean isOwner(String personId, String houseId) {
        if (StringUtils.isEmpty(personId) || StringUtils.isEmpty(houseId)) {
            return false;
        }

        List<TaShopKeeper> keeperList = iTaShopKeeperService.getByPerson(personId);
        if (null == keeperList || keeperList.size() < 1) {
            return false;
        }

        List<String> shopIdList = new ArrayList<>();
        for (TaShopKeeper keeper: keeperList) {
            shopIdList.add(keeper.getShopId());
        }

        QueryWrapper<TaHouse> queryWrapper = new QueryWrapper<TaHouse>()
                .eq("house_id", houseId)
                .in("shop_id", shopIdList)
                .eq("status", StatusConstant.NORMAL);

        return count(queryWrapper) > 0;
    }
}
